require 'test_helper'

class ChannelTest < EssentialTest
  include Essential::Messaging

  def test_retrieve
    assert_raises ArgumentError do
      channel = Channel.retrieve('')
    end

    assert_raises ArgumentError do
      Channel.retrieve(nil)
    end

    channel = Channel.retrieve('ch_p03Gjl8Uzn0RkZSpHXHnrw')
    assert_equal 'ch_p03Gjl8Uzn0RkZSpHXHnrw', channel.sid
    refute_nil channel.name
    refute_nil channel.created_at
    refute_nil channel.updated_at
  end

  def test_list
    channels = Channel.list

    assert channels.size > 1, 'must have more than one channel'

    first = channels[0]
    last  = channels[-1]

    refute_equal first.sid, last.sid
  end

  def test_update
    secondary = Channel.list.find do |ch|
      ch.name != 'default' && ch.name != ch.name.reverse
    end

    refute_nil secondary
    sid = secondary.sid
    original_name = secondary.name

    refute_equal 'default', secondary.name

    secondary.update(name: secondary.name.reverse)
    assert_equal sid, secondary.sid
    refute_equal original_name, secondary.name
    assert_equal original_name, secondary.name.reverse
  end

  def test_relations
    channel = Channel.retrieve('p03Gjl8Uzn0RkZSpHXHnrw')
    assert_equal Essential::Account.retrieve, channel.account

    subs = channel.subscribers
    refute_equal 0, subs.count, 'test requires at least one Subscriber'
    assert subs.all? {|s| Subscriber === s}

    mess = channel.messages
    refute_equal 0, mess.count, 'test requires at least one Message'
    assert mess.all? {|m| Message === m}
  end
end
