require 'test_helper'

class AlternateAuthenticationTest < EssentialTest
  def setup
    @account = Essential::Account.retrieve
    @alt_account = Essential::Account.retrieve(
      headers: {sid: 'eJ1WDpPBufJuDDYVO5xrKg', token: 'db72ed0fc101d65c5354834191d833e21a'}
    )

    refute_nil @alt_account, 'missing @alt_account: your dev setup needs help!'
    refute_equal @account.sid, @alt_account.sid, '@account and @alt_account must be different'
    refute_equal @account.token, @alt_account.token, '@account and @alt_account must be different'
  end

  def test_messages_account
    evaluate_relations(obj: 'ac.messages',
                       default: @account.messages,
                       alterate: @alt_account.messages)

    dm = Essential::Messaging::Message.list
    am = Essential::Messaging::Message.list(
      headers: {sid: @alt_account.sid, token: @alt_account.token}
    )

    evaluate_relations(obj: 'ac.messages', default: dm, alterate: am)
  end

  def test_messages_subscriber
    dsub = @account.messages.first.subscriber
    asub = @alt_account.messages.first.subscriber

    evaluate_relations(obj: 'sub.messages', default: dsub.messages, alterate: asub.messages)
  end

  def test_messages_channel
    dch = @account.messages.first.channel
    ach = @alt_account.messages.first.channel

    evaluate_relations(obj: 'ch.messages', default: dch.messages, alterate: ach.messages)
  end

  def test_messages_transport
    dtr = @account.messages.first.transport
    atr = @alt_account.messages.first.transport

    evaluate_relations(obj: 'tr.messages', default: dtr.messages, alterate: atr.messages)
  end

  def test_channels_account
    evaluate_relations(obj: 'ac.channels',
                       default: @account.channels,
                       alterate: @alt_account.channels)

    alts = Essential::Messaging::Channel.list(
      headers: {sid: @alt_account.sid, token: @alt_account.token}
    )
    evaluate_relations(obj: 'ac.channels',
                       default: Essential::Messaging::Channel.list,
                       alterate: alts)
  end

  def test_subscribers_account
    evaluate_relations(obj: 'ac.subscribers',
                       default: @account.subscribers,
                       alterate: @alt_account.subscribers)

    alts = Essential::Messaging::Subscriber.list(
      headers: {sid: @alt_account.sid, token: @alt_account.token}
    )
    evaluate_relations(obj: 'ac.subscribers',
                       default: Essential::Messaging::Subscriber.list,
                       alterate: alts)
  end

  def test_subscribers_channel
    dch = @account.subscribers.first.channel
    ach = @alt_account.subscribers.first.channel

    evaluate_relations(obj: 'ch.subscribers',
                       default: dch.subscribers,
                       alterate: ach.subscribers)
  end

  def test_subscribers_transport
    dtr = @account.subscribers.first.transport
    atr = @alt_account.subscribers.first.transport

    evaluate_relations(obj: 'tr.subscribers',
                       default: dtr.subscribers,
                       alterate: atr.subscribers)
  end

  def test_transports_account
    evaluate_relations(obj: 'ac.transports',
                       default: @account.transports,
                       alterate: @alt_account.transports)

    alts = Essential::Messaging::Transport.list(
      headers: {sid: @alt_account.sid, token: @alt_account.token}
    )
    evaluate_relations(obj: 'ac.transports',
                       default: Essential::Messaging::Transport.list,
                       alterate: alts)
  end

  def test_transports_channel
    dch = @account.transports.first.channel
    ach = @alt_account.transports.first.channel

    evaluate_relations(obj: 'ch.transports',
                       default: dch.transports,
                       alterate: ach.transports)
  end

  protected

  def evaluate_relations(obj:, default:, alterate:)
    refute_equal 0, default.size, format('missing default %s: your dev setup needs help!', obj)
    refute_equal 0, alterate.size, format('missing alterate %s: your dev setup needs help!', obj)

    default = Set.new(default.map(&:sid))
    alterate = Set.new(alterate.map(&:sid))

    refute_equal default, alterate
    assert_equal Set.new, (default & alterate), format('%s sids may not overlap', obj)
  end

end
